@@ -1,18 +1,39 @@
 import uritemplate  # pylint: disable=E0401
 from django.utils.encoding import smart_str
 from rest_framework.fields import Field
-from rest_framework.schemas.openapi import AutoSchema
+from rest_framework.request import Request
+from rest_framework.schemas.generators import EndpointEnumerator
+from rest_framework.schemas.openapi import AutoSchema, SchemaGenerator
 from rest_framework.serializers import BaseSerializer, Serializer
 from rest_framework.utils import formatting
 
-from .serializers import DetailSerializer, EmptySerializer, MockSerializer
-from .typing import Any, Container, Dict, ExternalDocs, HTTPMethod, List, Optional, Sequence, Type, Union
+from .serializers import EmptySerializer, MockSerializer
+from .typing import (
+    Any,
+    APIContact,
+    APIInfo,
+    APILicense,
+    APISchema,
+    Container,
+    Dict,
+    ExternalDocs,
+    HTTPMethod,
+    List,
+    Optional,
+    Sequence,
+    SerializerType,
+    Type,
+    Union,
+)
 from .utils import is_serializer_class
 
 
 __all__ = [
     "PipelineSchemaMixin",
     "PipelineSchema",
+    "VersionEndpointEnumerator",
+    "VersionSchemaGeneratorMixin",
+    "VersionSchemaGenerator",
     "convert_to_schema",
 ]
 
@@ -90,7 +111,7 @@ class PipelineSchemaMixin:
 
         components = {}
 
-        for serializer_class in (DetailSerializer, request_serializer_class, response_serializer_class):
+        for serializer_class in (request_serializer_class, response_serializer_class):
             serializer = self.view.initialize_serializer(serializer_class=serializer_class)
 
             if getattr(serializer, "many", False):
@@ -152,26 +173,28 @@ class PipelineSchemaMixin:
             responses.setdefault(200, ...)
 
         for status_code, info in responses.items():
-            serializer_class = DetailSerializer
-
-            if is_serializer_class(info):
-                serializer_class = info
-                info = serializer_class.__doc__ or ""
+            serializer_class: Optional[SerializerType] = None
 
             if info is ...:
                 serializer_class = self.view.get_serializer_class(output=True)
+                info = serializer_class.__doc__ or ""
 
                 if getattr(serializer_class, "many", False):
                     data.setdefault("204", self._get_no_result_schema())
 
+            elif is_serializer_class(info):
+                serializer_class = info
                 info = serializer_class.__doc__ or ""
 
-            serializer = self.view.initialize_serializer(serializer_class=serializer_class)
+            if serializer_class is not None:
+                serializer = self.view.initialize_serializer(serializer_class=serializer_class)
 
-            if isinstance(serializer, EmptySerializer):
-                status_code = 204
+                if isinstance(serializer, EmptySerializer):
+                    status_code = 204
 
-            response_schema = self._get_reference(serializer)
+                response_schema = self._get_reference(serializer)
+            else:
+                response_schema = {"schema": self._get_error_message_schema()}
 
             data[str(status_code)] = {
                 "content": {content_type: response_schema for content_type in response_media_types},
@@ -273,10 +296,17 @@ class PipelineSchemaMixin:
         return response_schema
 
     @staticmethod
-    def _get_no_result_schema():
+    def _get_no_result_schema(description: str = "No Results") -> Dict[str, Any]:
         return {
             "content": {"application/json": {"type": "string", "default": ""}},
-            "description": "No Results",
+            "description": description,
+        }
+
+    @staticmethod
+    def _get_error_message_schema(error_message: str = "Error message") -> Dict[str, Any]:
+        return {
+            "type": "object",
+            "properties": {"detail": {"type": "string", "default": error_message}},
         }
 
     def get_request_serializer(self, path: str, method: HTTPMethod) -> BaseSerializer:  # pylint: disable=W0613
@@ -306,3 +336,122 @@ class PipelineSchema(PipelineSchemaMixin, AutoSchema):
         self.external_docs = external_docs or self.external_docs
         self.prefix = prefix or self.prefix
         super().__init__(tags=tags, operation_id_base=operation_id_base, component_name=component_name)
+
+
+class VersionEndpointEnumerator(EndpointEnumerator):
+
+    url: str = ""
+
+    def get_path_from_regex(self, path_regex: str) -> str:
+        reg = super().get_path_from_regex(path_regex)
+        url = self.url
+
+        if reg.startswith("/"):
+            reg = reg[1:]
+        if url.startswith("/"):
+            url = url[1:]
+
+        if reg[: len(url)] == url:
+            return reg[len(url) :]
+
+        return reg
+
+
+class VersionSchemaGeneratorMixin(SchemaGenerator):
+
+    contact: APIContact = {}
+    license: APILicense = {}
+    terms_of_service: str = ""
+
+    endpoint_inspector_cls = VersionEndpointEnumerator
+
+    @classmethod
+    def with_info(
+        cls,
+        contact: Optional[APIContact] = None,
+        license: Optional[APILicense] = None,  # pylint: disable=redefined-builtin
+        terms_of_service: Optional[str] = None,
+    ) -> Type[SchemaGenerator]:
+        return type(  # type: ignore
+            cls.__name__,
+            (cls,),
+            {
+                "contact": contact or cls.contact,
+                "license": license or cls.license,
+                "terms_of_service": terms_of_service or cls.terms_of_service,
+            },
+        )
+
+    def get_info(self) -> APIInfo:
+        info: APIInfo = super().get_info()  # type: ignore
+        if self.contact:
+            info["contact"] = self.contact
+        if self.license:
+            info["license"] = self.license
+        if self.terms_of_service:
+            info["termsOfService"] = self.terms_of_service
+        return info
+
+    def get_schema(self, request: Optional[Request] = None, public: bool = False) -> APISchema:
+        schema: APISchema = super().get_schema(request, public)  # type: ignore
+
+        sorted_paths = {}
+        for path, operations in schema["paths"].items():
+            tag = list(operations.values())[0]["tags"][0]
+            sorted_paths.setdefault(tag, {})
+            sorted_paths[tag][path] = operations
+
+        schema["paths"] = {
+            path: operations
+            for endpoints in dict(sorted(sorted_paths.items())).values()
+            for path, operations in dict(sorted(endpoints.items())).items()
+        }
+
+        return schema
+
+
+class VersionSchemaGenerator(VersionSchemaGeneratorMixin, SchemaGenerator):
+    def __init__(
+        self,
+        title: Optional[str] = None,
+        url: Optional[str] = None,
+        description: Optional[str] = None,
+        patterns: Optional[List[str]] = None,
+        urlconf: Optional[str] = None,
+        version: Optional[str] = None,
+        contact: Optional[APIContact] = None,
+        license: Optional[APILicense] = None,  # pylint: disable=redefined-builtin
+        terms_of_service: Optional[str] = None,
+    ):
+        """Create a versioned schema.
+
+        :param title: The name of the API (required).
+        :param url: The root URL of the API schema. This option is not required unless
+                    the schema is included under path prefix.
+        :param description: Longer descriptive text.
+        :param patterns: A list of URLs to inspect when generating the schema.
+                         Defaults to the project's URL conf.
+        :param urlconf: A URL conf module name to use when generating the schema.
+                        Defaults to settings.ROOT_URLCONF.
+        :param version: The version of the API. Defaults to 0.1.0.
+        :param contact: API developer contact information.
+        :param license: API license information.
+        :param terms_of_service: API terms of service link.
+        """
+
+        if url and not url.startswith("/"):
+            url = "/" + url
+
+        super().__init__(
+            title=title,
+            url=url,
+            description=description,
+            patterns=patterns,
+            urlconf=urlconf,
+            version=version,
+        )
+
+        self.endpoint_inspector_cls.url = self.url or ""
+        self.contact = contact or self.contact
+        self.license = license or self.license
+        self.terms_of_service = terms_of_service or self.terms_of_service
